These are chronologically-order notes taken while working on that prototype to support dates (absolute and relative) in zodbtools' `<tidrange>`


# git's approxidate test "as is"

both python and go libraries do not support inputs like `1.month.ago`, so we preprocess to remove the `.`. (python's dateparser supports some formats though)

```
python prototype/proto.py
ERROR timespec: 10am noon expected time: 2009-08-29T12:00:00Z parsed time: 2009-08-30T10:00:00Z
ERROR timespec: last tuesday expected time: 2009-08-25T19:20:00Z parsed time: None
ERROR timespec: July 5th expected time: 2009-07-05T19:20:00Z parsed time: 2009-07-05T00:00:00Z
ERROR timespec: 06/05/2009 expected time: 2009-06-05T19:20:00Z parsed time: 2009-06-05T00:00:00Z
ERROR timespec: 06.05.2009 expected time: 2009-05-06T19:20:00Z parsed time: 2009-06-05T00:00:00Z
ERROR timespec: 2008-12-01 expected time: 2008-12-01T19:20:00Z parsed time: 2008-12-01T00:00:00Z
ERROR timespec: 2009-12-01 expected time: 2009-12-01T19:20:00Z parsed time: 2009-12-01T00:00:00Z

$ go run prototype/proto.go
ERROR timespec: 2.years.3.months.ago expected time: 2007-05-30T19:20:00Z parsed time: 2009-05-30T19:20:00Z
ERROR timespec: 10am noon expected time: 2009-08-29T12:00:00Z parsed time: 2009-08-30T10:00:00Z
ERROR timespec: 06/05/2009 expected time: 2009-06-05T19:20:00Z parsed time: 2009-06-05T00:00:00Z
ERROR timespec: 06.05.2009 expected time: 2009-05-06T19:20:00Z parsed time: 2009-06-05T00:00:00Z
ERROR timespec: Jun 6, 5AM expected time: 2009-06-06T05:00:00Z parsed time: 0000-06-06T00:00:05Z
ERROR timespec: 5AM Jun 6 expected time: 2009-06-06T05:00:00Z parsed time: 0000-06-01T00:00:05Z
ERROR timespec: 2008-12-01 expected time: 2008-12-01T19:20:00Z parsed time: 2008-12-01T00:00:00Z
ERROR timespec: 2009-12-01 expected time: 2009-12-01T19:20:00Z parsed time: 2009-12-01T00:00:00Z

```

* go's `when` does not support "chains" like `2.years.3.months.ago`.
* `10am noon` is ambigious, we don't want so support it.
* date only `2008-12-01` should not retain "current" hour:minute

# after adjusting the input

```
python prototype/proto.py
ERROR timespec: last tuesday expected time: 2009-08-25T19:20:00Z parsed time: None
ERROR timespec: 06.05.2009 expected time: 2009-05-06T00:00:00Z parsed time: 2009-06-05T00:00:00Z

go run prototype/proto.go
ERROR timespec: 2.years.3.months.ago expected time: 2007-05-30T19:20:00Z parsed time: 2009-05-30T19:20:00Z
ERROR timespec: July 5th expected time: 2009-07-05T00:00:00Z parsed time: 2009-07-05T19:20:00Z
ERROR timespec: 06.05.2009 expected time: 2009-05-06T00:00:00Z parsed time: 2009-06-05T00:00:00Z
ERROR timespec: Jun 6, 5AM expected time: 2009-06-06T05:00:00Z parsed time: 0000-06-06T00:00:05Z
ERROR timespec: 5AM Jun 6 expected time: 2009-06-06T05:00:00Z parsed time: 0000-06-01T00:00:05Z

```

# after disabling unsupported inputs

```
python prototype/proto.py
go run prototype/proto.go
```

no more errors reported.


# but is all this really a good idea ?

As we could see go and python have small differences, but it's OK.

One concern is that parsers might not report errors on invalid input, as they support a wide range of formats.
dateparser has a `STRICT_PARSING` option, but it does not seem to be so effective after a very short test.


With valid inputs, current implementations de-facto support many formats, so for users who don't read the manual, it seem the tool support "any format and it will magically work", but sometimes it might not do what they think.

The original need was to support using date in tidrange, without having to "calculate" the tid from hash.

Relative dates are a cool feature, but I feel the most common use cases would be to use absolute dates.
If we only supported only one simple format (ISO8601), we would be able to reliably detect and refuse invalid inputs.

That said, this is a tool for power users, so maybe that's OK as is. All these arguments exists for any powerful but dangerous tools such as using chainsaw to cut wood faster or cars to move faster, so maybe we should not worry here.

While thinking about this, I checked unix `date` command and realized it also supports "clever" time parsing, for example:

```bash
date -d yesterday --iso-8601=seconds  --utc
```

with `prototype/proto_unix_date.py` to check support for the reference dates:

```
(env) $ python prototype/proto_unix_date.py
ERROR timespec: 1985-04-12T23:20:50.52Z expected time: 1985-04-12T23:20:50.520000Z parsed time: 1985-04-12T23:20:50Z
ERROR timespec: 6am yesterday expected time: 2009-08-29T04:00:00Z parsed time: 2009-08-29T06:00:00Z
ERROR timespec: 6pm yesterday expected time: 2009-08-29T16:00:00Z parsed time: 2009-08-29T18:00:00Z
ERROR timespec: 3:00 expected time: 2009-08-30T01:00:00Z parsed time: 2009-08-30T03:00:00Z
ERROR timespec: 15:00 expected time: 2009-08-30T13:00:00Z parsed time: 2009-08-30T15:00:00Z
ERROR timespec: noon today expected time: 2009-08-30T10:00:00Z parsed time: date: invalid date 'noon today'
ERROR timespec: noon yesterday expected time: 2009-08-29T10:00:00Z parsed time: date: invalid date 'noon yesterday'
ERROR timespec: 6AM, June 7, 2009 expected time: 2009-06-07T04:00:00Z parsed time: date: invalid date '6AM, June 7, 2009'
```

Another completly different approach could be:
 * `<tidrange>` supports only hex tid
 * new subcommands are added `tid-from-iso8601` and `iso8601-from-tid` (the later is not directly needed here, but could be convenient as well)

Reports on relative dates could be obtained with combination of simple commands for which , wh for example:
```bash
zodb analyze Data.fs $(zodb tid-from-iso8601 $(date -d "3 weeks ago" --iso-8601=seconds))..$(zodb tid-from-iso8601 $(date -d "yesterday" --iso-8601=seconds))
```



# TODO

* [X] spec with "local timezone" properly.
* [X] make sure go's proto handling of timezone is correct (it's copy pasted from examples)