"""Extends tidrange-formats.txt with tids
"""

from __future__ import print_function
import argparse
import dateparser

from ZODB.TimeStamp import TimeStamp
from zodbtools.util import ashex


def toTID(dt):
    return ashex(
        TimeStamp(
            dt.year,
            dt.month,
            dt.day,
            dt.hour,
            dt.minute,
            dt.second + dt.microsecond / 1000000.0,
        ).raw()
    )


parser = argparse.ArgumentParser(description="Prototype script for tidrange parsing")
parser.add_argument(
    "--formats",
    default="./zodbtools/test/testdata/tidrange-formats.txt",
    help="Path of tidrange-formats.txt",
)
args = parser.parse_args()

dateparser_settings = {"TO_TIMEZONE": "UTC", "RETURN_AS_TIMEZONE_AWARE": True}


with open(args.formats, "r") as f:
    for line in f:
        line = line.strip()
        # skip comments or empty lines
        if not line or line.startswith("#"):
            if not (line.startswith("#1") or line.startswith("#2")):
                print(line)
                continue
        reference, _, timespec = line.split(" ", 2)

        parsed_time = dateparser.parse(
            reference[1:] if reference.startswith("#") else reference,
            settings=dateparser_settings,
        )
        print(reference, toTID(parsed_time), timespec)
