# check our format with unix date command
from __future__ import print_function
import argparse
import subprocess

parser = argparse.ArgumentParser(description="Prototype script for tidrange parsing")
parser.add_argument(
    "--formats",
    default="./zodbtools/test/testdata/tidrange-formats.txt",
    help="Path of tidrange-formats.txt",
)
if 0:
    # faketime's $FAKETIME and $FAKETIME_FMT does not seem to support timezones,
    # so this is not configurable as command line args.
    parser.add_argument(
        "--timezone",
        default="Europe/Paris",
        help="Timezone aka `America/Los_Angeles` formatted time-zone",
    )
    parser.add_argument(
        "--relative-time-base",
        default="2009-08-30T19:20:00Z",
        help="Consider relative time as relative to this RFC3339 formatted time",
    )
args = parser.parse_args()

args.timezone = "Europe/Paris"
args.relative_time_base = "2009-08-30T21:20:00Z"

with open(args.formats, "r") as f:
    for line in f:
        line = line.strip()
        # skip comments or empty lines
        if not line or line.startswith("#"):
            continue

        reference, _, timespec = line.split(" ", 2)
        preprocessed_timespec = timespec
        if "ago" in timespec:
            preprocessed_timespec = timespec.replace(".", " ").replace("_", " ")

        try:
            parsed = subprocess.check_output(
                ("date", "-d", preprocessed_timespec, "--iso-8601=seconds", "--utc"),
                stderr=subprocess.STDOUT,
                env={
                    "TZ": args.timezone,
                    "FAKETIME": args.relative_time_base,
                    "FAKETIME_FMT": "%Y-%m-%dT%H:%M:%S%z",
                    # https://github.com/wolfcw/libfaketime
                    "LD_PRELOAD": "/srv/slapgrid/slappart8/srv/runner/project/zodbtools/faketime/libfaketime/src/libfaketime.so.1",
                }).strip().decode('utf-8')
            # Normalize the output, as date command outputs timzone as +00:00 but our reference times use Z
            assert parsed.endswith('+00:00')
            parsed = parsed[:-len('+00:00')] + 'Z'
        except subprocess.CalledProcessError as e:
            parsed = e.output.strip()

        if parsed != reference:
             print(
                "ERROR timespec:",
                timespec,
                # "( preprocessed as", preprocessed_timespec, ")",
                "expected time:",
                reference,
                "parsed time:",
                parsed
            )